#!/bin/bash

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# This file is creationScript_multiProfile_v1.0.1.sh
# Modifications by Jason Quinlan  - j.quinlan@cs.ucc.ie - 09th March 2018

# version 1.0.1

# This is a bash script for creating DASH profiles for our 4K dataset

# resolutions
ourRez=( 3840x2160 1920x1080 1280x720 736x414 640x360 512x288 384x216 320x180 )
resList=${ourRez[@]}

# bitrates to use
# 3840x2160		40000   25000   15000
# 1920x1080		4300	3850
# 1280x720		3000	2350
# 736x414		1750
# 640x360		1050
# 512x288		750		560
# 384x216		375
# 320x180		235
# indexs from 0 to 9, not 1 to 10
ourbitrates=( 40000 25000 15000 4300 3850 3000 2350 1750 1050 750 560 375 235 )
bitrateList=${ourbitrates[@]}

# representation rate to resolution rate array
# 3 bitrates @ 2160
# 2 bitrates @ 1080
# 2 bitrates @  720
# 2 bitrates @  480
# 2 bitrates @  384
# 1 bitrates @  288
# 1 bitrates @  272
# ourReprate=(3 2 2 1 1 2 1 1)
ourReprate=(3 2 2 1 1 2 1 1)

# ====== The following items will define our folder ========

# encoders
# encoders=( x264 x265 )
encoders=( x265 )

# file location
dash_location="DASH_Files"
mp4_location="MP4_Files"
script_location="Script_Files"
content_location="4K_non_copyright_dataset"

echo "Create Directory Structure"

# DASH segment size in seconds
# ourSec=( 2 4 6 8 10 )
ourSec=( 2 4 6 8 10 )

# *********  BELOW - THESE MUST BE CHANGED  ************
# fps - THIS MUST BE CHANGED - 60 for bbb, 24 for the other two clips
myfps=24

# file location
mkv_location="Video_Files"
# *********  ABOVE - THESE MUST BE CHANGED  ************

# folder structure needed:
# <4K_non_copyright_dataset>
#    <num_sec>_sec
#        <encoder>
#            <folder> - using <Video_Files> for clip names
#                <mp4_location>
#            
# <Video_Files>
#    <folder> - clip to DASH
#    
# <DASH_Script_Files>


# DASH profiles
DASH_profiles=( "onDemand" "live" "main" "full" )

# lets move to the content folder first
cd ${content_location}

# ==================== Let's Begin ===========================
# for each DASH profile
for DASH_profile in "${DASH_profiles[@]}"
do

    # for each resolution, we create a yuv file
    for num_sec in "${ourSec[@]}"
    do

        chunk_size=$(( $num_sec * 1000  ))


        # "{mkv_location}/An_Idiot_Abroad"
        for folder in `ls ../${mkv_location}`
        do
	
			echo "Pre-processing" ${DASH_profile}

		    inputFilePre=${folder}

			# lets move the newly created folders and their contents to a new folder
			outPutFolderName=${folder}

			# print folder name
			echo "${folder} - is now being converted with a segment duration of ${num_sec}"

			echo "Create DASH files"
			# step 2: create the encoded files
			for encoder in "${encoders[@]}"
			do

				# onDemand does not do segments so do not encode
				if ! [ $DASH_profile == "onDemand" ]
						then
	
					# lets create a segment duration style structure
					mkdir -p ./${num_sec}_sec/${encoder}/${outPutFolderName}/${dash_location}/${DASH_profile}
					cd ./${num_sec}_sec/${encoder}/${outPutFolderName}/${dash_location}/${DASH_profile}

                    # some counters for our folder structure
                    bitrate_counter=0
                    bitrate_index=0
					output_dash_string=""
                
                    # create the folder structure for our DASH files
                    #for each of the resolutions
                    for our_rez in "${ourRez[@]}"
                    do
                        counter=0

                        #echo ${ourReprate[${bitrate_index}]}

                        # for our_rez in "${ourRez[@]}"
                        while [ $counter -lt ${ourReprate[${bitrate_index}]} ]
                        do
					        mkdir -p ./${our_rez}/${ourbitrates[bitrate_counter]}kbps

                            # echo "./${our_rez}/${ourbitrates[bitrate_counter]}kbps"

							# lets concat the dash_output string
							output_dash_string+="../../${mp4_location}/${our_rez}/${ourbitrates[bitrate_counter]}kbps/${inputFilePre}_${our_rez}_${myfps}fps_${ourbitrates[bitrate_counter]}kbps.mp4 "

                            # increment our 'counter' counter
                            counter=$[$counter+1]

                            # increment our 'bitrate_counter' counter
                            bitrate_counter=$[$bitrate_counter+1]

                        done

                        # increment our 'bitrate_index' counter
                        bitrate_index=$[$bitrate_index+1]

                    done

					# # This code creates an mpd file which should contains the same encoding codec for each representation
					# -rap has been removed - issues with x265
				    MP4Box -profile ${DASH_profile} -rap -dash $chunk_size -frag $chunk_size -segment-name %s_segment -url-template -out ${inputFilePre}_enc_${encoder}_dash.mpd ${output_dash_string}

			        # move the newly created DASH files into their respective folders
			        for rez in "${ourRez[@]}"
			        do
				        for bitrate in `ls ${rez}`
				        do

				        checkOutput=${rez}_${myfps}fps_${bitrate}

				        mv *${checkOutput}* ./${rez}/${bitrate}/

				        done
			        done

		            # add the folder strucure created above to the mpd file
		            awk -f ../../../../../../${script_location}/reset_MPD_folder_structure.awk ${inputFilePre}_enc_${encoder}_dash.mpd > ${inputFilePre}_enc_${encoder}_dash_temp.mpd
		            rm ${inputFilePre}_enc_${encoder}_dash.mpd
		            cp ${inputFilePre}_enc_${encoder}_dash_temp.mpd ${inputFilePre}_enc_${encoder}_dash.mpd
		            rm ${inputFilePre}_enc_${encoder}_dash_temp.mpd

			        cd ../../../../../

			        echo

		        fi

		        # live does not do byte range so do not encode
		        if ! [ $DASH_profile == "live" ]
				then

		            # lets also create a byte-range style structure
		            mkdir -p ./${num_sec}_sec/${encoder}/${outPutFolderName}/${dash_location}/${DASH_profile}_byte_range
			        cd ./${num_sec}_sec/${encoder}/${outPutFolderName}/${dash_location}/${DASH_profile}_byte_range

			        # some counters for our folder structure
                    bitrate_counter=0
                    bitrate_index=0
					output_dash_string=""
                
                    # create the folder structure for our DASH files
                    #for each of the resolutions
                    for our_rez in "${ourRez[@]}"
                    do
                        counter=0

                        # echo ${ourReprate[${bitrate_index}]}

                        # for our_rez in "${ourRez[@]}"
                        while [ $counter -lt ${ourReprate[${bitrate_index}]} ]
                        do
					        mkdir -p ./${our_rez}/${ourbitrates[bitrate_counter]}kbps

                            # echo "./${our_rez}/${ourbitrates[bitrate_counter]}kbps"

							# lets concat the dash_output string
							output_dash_string+="../../${mp4_location}/${our_rez}/${ourbitrates[bitrate_counter]}kbps/${inputFilePre}_${our_rez}_${myfps}fps_${ourbitrates[bitrate_counter]}kbps.mp4 "

                            # increment our 'counter' counter
                            counter=$[$counter+1]

                            # increment our 'bitrate_counter' counter
                            bitrate_counter=$[$bitrate_counter+1]

                        done

                        # increment our 'bitrate_index' counter
                        bitrate_index=$[$bitrate_index+1]

                    done

			        # # This code creates an mpd file which should contains the same encoding codec for each representation
		            # -rap has been removed - issues with x265
			        MP4Box -profile ${DASH_profile} -rap -dash $chunk_size -frag $chunk_size -out ${inputFilePre}_enc_${encoder}_dash.mpd ${output_dash_string} 

			        # move the newly created DASH files into their respective folders
			        for rez in "${ourRez[@]}"
			        do
				        for bitrate in `ls ${rez}`
				        do

				        checkOutput=${rez}_${myfps}fps_${bitrate}

				        mv *${checkOutput}* ./${rez}/${bitrate}/

				        done
			        done

		            # add the folder strucure created above to the mpd file
		            awk -f ../../../../../../${script_location}/reset_byte_range_MPD_folder_structure.awk ${inputFilePre}_enc_${encoder}_dash.mpd > ${inputFilePre}_enc_${encoder}_dash_temp.mpd
		            rm ${inputFilePre}_enc_${encoder}_dash.mpd
		            cp ${inputFilePre}_enc_${encoder}_dash_temp.mpd ${inputFilePre}_enc_${encoder}_dash.mpd
		            rm ${inputFilePre}_enc_${encoder}_dash_temp.mpd

			        cd ../../../../../
	
			        echo

		        fi

	        done
	
        done

    done

done


